<?php
/*
Plugin Name: Disclose-Secret
Plugin URI: http://bluesome.net/post/2006/01/01/121/
Description: Only shows Wordpress content to other users if they meet certain access criteria.
Version: 2.0.1
Author: S&ouml;ren Weber
Author URI: http://bluesome.net

This work was initally derived from Filipe Fortes Post Levels plugin (http://fortes.com/)
but now has influences of the many other restriction plugins out there, as eg. ViewLevels
and ViewLevels2, Private Categories, etc.
*/

require_once(dirname(__FILE__).'/manager.php');

// ----------------------------------------------------------------------------
// tempalate functions
// ----------------------------------------------------------------------------

// Whether the current post is private; means marked private
// and does not have any restrictions defined for
function is_private($post_id = NULL)
{
	if (!isset($post_id))
	{
		global $post;
		$post_id = $post->ID;
	}

	global $g_disclosesecret_manager;
	if (!isset($g_disclosesecret_manager))
		$g_disclosesecret_manager = new DiscloseSecret_Manager();

	$cache = $g_disclosesecret_manager->get_cache();
	if (!$cache)
	{
		// assure functionality even if plugin is disabled due to a version conflict
		$post = get_post($this->m_post_id);
		return $post->post_status == 'private';
	}

	$postmeta = $cache->get_postmeta($post_id);
	return $postmeta->is_private();
}

// Whether the current post is protected; means marked private
// but has restrictions defined for
function is_protected($post_id = NULL)
{
	if (!isset($post_id))
	{
		global $post;
		$post_id = $post->ID;
	}

	global $g_disclosesecret_manager;
	if (!isset($g_disclosesecret_manager))
		$g_disclosesecret_manager = new DiscloseSecret_Manager();

	$cache = $g_disclosesecret_manager->get_cache();
	if (!$cache)
		return false;

	$postmeta = $cache->get_postmeta($post_id);
	return $postmeta->is_protected();
}

// ----------------------------------------------------------------------------
// pluggable functions
// ----------------------------------------------------------------------------

// Override WP's get_currentuserinfo in order to do the login
// via HTTP auth. This code is copied from WP 2.0.4's get_currentuserinfo
if ($_GET[DiscloseSecret_POST_HTTP_AUTH] == DiscloseSecret_POST_HTTP_AUTH_DO)
{
	function get_currentuserinfo()
	{
		// Use HTTP auth instead of cookies
		global $current_user;

		if (!empty($current_user))
			return;

		if (!isset($_SERVER['PHP_AUTH_USER']) || !isset($_SERVER['PHP_AUTH_PW'])
			|| !wp_login($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']))
		{
			header('WWW-Authenticate: Basic realm="'. get_bloginfo('name'). '"');
			header('HTTP/1.0 401 Unauthorized');

			die(__('Access denied: Incorrect credentials supplied.', DiscloseSecret_PLUGIN_ID));
		}

		$user_login = $_SERVER['PHP_AUTH_USER'];
		wp_set_current_user(0, $user_login);
	}
}

// ----------------------------------------------------------------------------
// main
// ----------------------------------------------------------------------------

global $g_disclosesecret_manager;
if (!isset($g_disclosesecret_manager))
	$g_disclosesecret_manager = new DiscloseSecret_Manager();

?>