<?php

require_once(dirname(__FILE__).'/cache.php');
require_once(dirname(__FILE__).'/config.php');
require_once(dirname(__FILE__).'/migrator.php');

// ----------------------------------------------------------------------------
// the DiscloseSecret_Interface class displays the various user interface
// parts, as the plugin configuration panel or the controls to set restrictions
// at a post
// ----------------------------------------------------------------------------

// use this guard to avoid error messages in WP admin panel if plugin
// is disabled because of a version conflict but you still try to reload
// the plugins admin interface
if (!class_exists('DiscloseSecret_Interface')) :
class DiscloseSecret_Interface
{
	var $m_cache;
	var $m_migrator;

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	// Sets up the Disclose-Secret admin menu
	function DiscloseSecret_Interface(&$cache)
	{
		$this->m_cache = $cache;

		add_filter('admin_menu', array(&$this, 'filter_admin_menu'));
	}

	function filter_admin_menu()
	{
		// delay initialization until the admin framework is loaded
		$this->m_migrator = new DiscloseSecret_Migrator();

		add_submenu_page('options-general.php',
			DiscloseSecret_PLUGIN_NAME,
			DiscloseSecret_PLUGIN_NAME,
			'edit_others_posts', __FILE__,
			array(&$this, 'submenu_page_option_general'));

		add_filter('manage_posts_custom_column', array(&$this, 'filter_manage_posts_custom_column'));
		add_filter('manage_posts_columns', array(&$this, 'filter_manage_posts_columns'));
		add_filter('dbx_post_advanced', array(&$this, 'filter_dbx_post_advanced'));
	}

	// ----------------------------------------------------------------------------
	// tools
	// ----------------------------------------------------------------------------

	function print_feed_privacy($feed_options, $feed_privacy)
	{
		foreach ($feed_options as $key => $value)
		{
			$selected = "";
			if ($key == $feed_privacy)
				$selected = " selected='yes'";
?>
			<option value="<?php echo $key; ?>"<?php echo $selected; ?>><?php echo $value; ?></option>
<?php
		}
	}

	// ----------------------------------------------------------------------------
	// admin interface
	// ----------------------------------------------------------------------------

	// Disclose-Secret configuration page
	function submenu_page_option_general()
	{
		global $wpdb;

		// executing form actions
		$option = $this->m_cache->get_option();
		if (isset($_POST[DiscloseSecret_ACTION_UPDATE_OPTIONS]))
		{
			check_admin_referer();
			$option->set_from_POST();
			$option->save();
			echo '<div id="message" class="updated fade"><p><strong>'.
				__('Options saved.', DiscloseSecret_PLUGIN_ID) . "</strong></p></div>\n";
		}

		$post_levels_installed = $this->m_migrator->migrate_post_levels(
			isset($_POST[DiscloseSecret_ACTION_MIGRATE_POST_LEVELS]));
		$view_levels_installed = $this->m_migrator->migrate_view_levels(
			isset($_POST[DiscloseSecret_ACTION_MIGRATE_VIEW_LEVELS]));
		$private_categories_installed = $this->m_migrator->migrate_private_categories(
			isset($_POST[DiscloseSecret_ACTION_MIGRATE_PRIVATE_CATEGORIES]));
		if ($post_levels_installed == DiscloseSecret_MIGRATION_DONE
			|| $view_levels_installed == DiscloseSecret_MIGRATION_DONE
			|| $private_categories_installed == DiscloseSecret_MIGRATION_DONE)
		{
			echo '<div id="message" class="updated fade"><p><strong>'.
				__('Plugin migrated', DiscloseSecret_PLUGIN_ID) . "</strong></p></div>\n";
		}
?>
	<div class="wrap">
		<h2><?php _e('Disclose-Secret Options', DiscloseSecret_PLUGIN_ID); ?></h2>
		<p><?php _e("Disclose-Secret shows private posts to other users if the user meets at least one of the post's restriction settings. Access to posts can be individually restricted on the 'Write Post' page per user, role or capability.", DiscloseSecret_PLUGIN_ID); ?></p>
		<form action="" method="post" id="<?php echo DiscloseSecret_POST_FORM_ID; ?>">
			<fieldset class="options">
				<legend><?php _e('Newsfeed options', DiscloseSecret_PLUGIN_ID); ?></legend>
				<p><?php _e("With the newsfeed options you can configure the behaviour of your newsfeeds when they are read by logged in users. ", DiscloseSecret_PLUGIN_ID); ?></p>
				<table class="editform optiontable">
					<tr valign="top">
						<th scope="row"><?php _e('Default Feed Privacy:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<select name="<?php echo DiscloseSecret_POST_FEED_PRIVACY; ?>" id="<?php echo DiscloseSecret_POST_FEED_PRIVACY; ?>">
<?php $this->print_feed_privacy($option->get_feed_options(), $option->get_feed_privacy()); ?>
							</select>
							<br />
							<?php _e("For logged in users reading feeds via HTTP authentication, this decides how much content of a protected or private post should be shown. Because newsfeeds may get cached by newsreader programs, it may be preferable to hide some or all potions of a post in the newsfeeds to avoid exposing content.", DiscloseSecret_PLUGIN_ID); ?>

						</td>
					</tr>

					<tr valign="top">
						<th scope="row"><?php _e('Feed Teaser:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<textarea name="<?php echo DiscloseSecret_POST_FEED_TEASER; ?>" id="<?php echo DiscloseSecret_POST_FEED_TEASER; ?>" cols="60" rows="4" style="width: 98%; font-size: 12px;" class="code"><?php echo htmlspecialchars($option->get_feed_teaser(), ENT_QUOTES); ?></textarea>
							<?php _e('Define the teaser text, that is shown to logged in users accessing your newsfeed in case not the full content or the excerpt is shown. This text will show up for each protected or the readers private posts. You can use the following placeholders for certain dynamic text:', DiscloseSecret_PLUGIN_ID); ?>

							<ul>
								<li><?php echo DiscloseSecret_PLACEHOLDER_PERMALINK; ?> - causing the permalink URL to be inserted</li>
							</ul>
						</td>
					</tr>

					<tr valign="top">
						<th scope="row"><?php _e('Rewrite Feed Links:', DiscloseSecret_PLUGIN_ID); ?></th>
						<td>
							<label for="<?php echo DiscloseSecret_POST_REWRITE_FEED_LINK; ?>">
								<input type="checkbox" name="<?php echo DiscloseSecret_POST_REWRITE_FEED_LINK; ?>" id="<?php echo DiscloseSecret_POST_REWRITE_FEED_LINK; ?>" value="true" <?php if ($option->get_rewrite_feed_link()) : ?>checked="checked" <?php endif; ?>/>
								<?php echo sprintf(__('For logged in users, automatically appends <code>%s=%s</code> to the query string of feed links to support HTTP Authentication. This for convenience to your readers.', DiscloseSecret_PLUGIN_ID), DiscloseSecret_POST_HTTP_AUTH, DiscloseSecret_POST_HTTP_AUTH_DO); ?>

							</label>
						</td>
					</tr>

				</table>
			</fieldset>

			<p class="submit">
				<input type="submit" name="<?php echo DiscloseSecret_ACTION_UPDATE_OPTIONS; ?>" value="<?php _e('Update Options') ?> &raquo;" />
			</p>
		</form>
	</div>
<?php if ($post_levels_installed == DiscloseSecret_MIGRATION_SKIPPED) : ?>

	<div class="wrap">
		<h2><?php _e('Migration from Post Levels', DiscloseSecret_PLUGIN_ID); ?></h2>
		<p><?php _e('Disclose-Secret has found that your blog used a version of the <a href="http://fortes.com/projects/wordpress/postlevels">Post Levels</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your post levels to post restrictions by pressing the button below.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('The migration will either convert the previous set post_level to equivalent &quot;level_X&quot; capabilities or in case you have used Post Levels &quot;Migrate user levels&quot; feature it will convert the used post levels to a set of newly created &quot;read_level_X&quot; capabilites (where X is the number of the legacy user level). In case of newly created capabilities these will be assigned in equivalent manner to the users as they were previously assigend by the Post Levels plugin. Additionally a new &quot;Post Levels Administrator&quot; role will be created were all of the newly created capabilities been assigned to. These admin role is not yet assigned to any user after migration. You are free to delete it if you do not want to use it. It is just meant to simplify role/capability management after migration.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Summary: After migration your blog behaves the same way as with Post Levels but is configured differently. You also should consider to install a Role Manager. See the common plugin repositories for implementations.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Migration was tested with Post Levels version 1.0 but should work with all previous versions.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><strong><?php _e('Note, that migration can not be undone by this plugin once you have pressed the button below.', DiscloseSecret_PLUGIN_ID); ?></strong></p>
		<form action="" method="post" id="<?php echo DiscloseSecret_POSTLEVELS_POST_FORM_ID; ?>">
			<p class="submit">
				<input type="submit" name="<?php echo DiscloseSecret_ACTION_MIGRATE_POST_LEVELS; ?>" value="<?php _e('Migrate Post Levels &raquo;', DiscloseSecret_PLUGIN_ID); ?>" />
			</p>
		</form>
	</div>
<?php endif; ?>
<?php if ($view_levels_installed == DiscloseSecret_MIGRATION_SKIPPED) : ?>

	<div class="wrap">
		<h2><?php _e('Migration from View Levels', DiscloseSecret_PLUGIN_ID); ?></h2>
		<p><?php _e('Disclose-Secret has found that your blog used a version of the <a href="http://blog.firetree.net/2005/08/25/viewlevel-20/">View Levels</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your view levels to post restrictions by pressing the button below.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Summary: After migration your blog\'s already published posts are converted to be protected by Disclose-Secret according to your previous View Levels configuration. Generally Disclose-Secret is more restrictive to your protected posts. E.g. they will not be published at an update service. Remember that there is no default restriction applied to new posts anymore. You also should consider to install a Role Manager. See the common plugin repositories for implementations.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Migration was tested with ViewLevel2 version 2.0.0 but should work with all previous versions including ViewLevel.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><strong><?php _e('Note, that migration can not be undone by this plugin once you have pressed the button below.', DiscloseSecret_PLUGIN_ID); ?></strong></p>
		<form action="" method="post" id="<?php echo DiscloseSecret_VIEWLEVELS_POST_FORM_ID; ?>">
			<p class="submit">
				<input type="submit" name="<?php echo DiscloseSecret_ACTION_MIGRATE_PRIVATE_VIEW_LEVELS; ?>" value="<?php _e('Migrate View Levels &raquo;', DiscloseSecret_PLUGIN_ID); ?>" />
			</p>
		</form>
	</div>
<?php endif; ?>
<?php if ($private_categories_installed == DiscloseSecret_MIGRATION_SKIPPED) : ?>

	<div class="wrap">
		<h2><?php _e('Migration from Private Categories', DiscloseSecret_PLUGIN_ID); ?></h2>
		<p><?php _e('Disclose-Secret has found that your blog used a version of the <a href="http://gregelin.com/archives/?page_id=2">Private Categories</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your private categories to post restrictions by pressing the button below.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Summary: After migration your blog behaves the same way as with Private Categories but is configured differently. Now the author has to take care to restrict the post when he saves it instead of the post being automatically protected by choosing the right categories. You also should consider to install a Role Manager. See the common plugin repositories for implementations.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Migration was tested with Private Categories 2 version 0.2.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><strong><?php _e('Note, that migration can not be undone by this plugin once you have pressed the button below.', DiscloseSecret_PLUGIN_ID); ?></strong></p>
		<form action="" method="post" id="<?php echo DiscloseSecret_PRIVATECATEGORIES_POST_FORM_ID; ?>">
			<p class="submit">
				<input type="submit" name="<?php echo DiscloseSecret_ACTION_MIGRATE_PRIVATE_CATEGORIES; ?>" value="<?php _e('Migrate Private Categories &raquo;', DiscloseSecret_PLUGIN_ID); ?>" />
			</p>
		</form>
	</div>
<?php endif; ?>
<?php
	}

	// ----------------------------------------------------------------------------
	// write post interface
	// ----------------------------------------------------------------------------

	function print_restriction_form($restriction_options, $restrictions,
		$post_key, $meta_key, $caption, $help, $legend)
	{
?>
<div class="dbx-box-wrapper">
	<fieldset id="<?php echo $meta_key; ?>" class="dbx-box">
		<div class="dbx-handle-wrapper">
			<h3 class="dbx-handle"><?php echo $caption; ?></h3>
		</div>
		<div class="dbx-content-wrapper">
			<div class="dbx-content">
				<p><?php echo $help; ?></p>
				<p><?php echo $legend; ?></p>
<?php
		foreach ($restriction_options as $id => $item)
		{
			$hint = '';
			if ($item[DiscloseSecret_INDEX_CAN_SEE_IMPLICIT])
				$hint .= ' [*]';
			if ($item[DiscloseSecret_INDEX_CAN_SEE_PROTECTED])
				$hint .= ' [!]';

			$mark = '';
			if (is_array($restrictions) && in_array($id, $restrictions))
			{
				$mark = '" checked="checked"';
			}

			$name = ' '. $item[DiscloseSecret_INDEX_NAME];
			if (!empty($hint) || !empty($mark))
			{
				$hint = '<strong class="current">'. $hint;
				$name .= ' </strong>';
			}
			$mark .= '" />';
?>
				<label style="float: left; width: 15em;"><input type="checkbox" name="<?php echo $post_key. $id; ?>" id="<?php echo $post_key. $id. $mark. $hint. $name; ?></label>
<?php
		}
?>
				<div style="clear: both;"></div>
			</div>
		</div>
	</fieldset>
</div>
<?php
	}

	function print_cap_form($postmeta)
	{
		$option = $this->m_cache->get_option();
		$this->print_restriction_form($option->get_caps(), $postmeta->get_caps(),
			DiscloseSecret_POST_CAP, DiscloseSecret_META_CAP,
			__('Access by capability', DiscloseSecret_PLUGIN_ID),
			__('Mark one or more capability that should be used to restrict access to this post. Only users with at least one of the highlighted capabilities will be able to view this post. The post will automatically marked as private and will not be seen by the public if you restrict the access.', DiscloseSecret_PLUGIN_ID),
			__('<ul><li>Users having a highlighted <strong class="current">capability</strong> written in bold will at least be able to read your protected and/or private posts.</li><li>Users having a capability marked with an exclamation mark <strong class="current">[!]</strong> due to Wordpress will <strong>always</strong> be able to read your protected or private posts, regardless of the settings you will do on this form. You can not change this behaviour.</li></ul>', DiscloseSecret_PLUGIN_ID));
	}

	function print_role_form($postmeta)
	{
		$option = $this->m_cache->get_option();
		$this->print_restriction_form($option->get_roles(), $postmeta->get_roles(),
			DiscloseSecret_POST_ROLE, DiscloseSecret_META_ROLE,
			__('Access by role', DiscloseSecret_PLUGIN_ID),
			__('Mark one or more role that should be used to restrict access to this post. Only users with at least one of the highlighted roles will be able to view this post. The post will automatically marked as private and will not be seen by the public if you restrict the access.', DiscloseSecret_PLUGIN_ID),
			__('<ul><li>Users having a highlighted <strong class="current">role</strong> written in bold will at least be able to read your protected and/or private posts.</li><li>Users having a role marked with an exclamation mark <strong class="current">[!]</strong> due to Wordpress will <strong>always</strong> be able to read your protected or private posts, regardless of the settings you will do on this form.</li><li>Users having a role marked with an asterisk mark <strong class="current">[*]</strong> will be able to read your protected posts, because of settings you did for &quot;Access by capability&quot;.</li></ul>', DiscloseSecret_PLUGIN_ID));
	}

	function print_user_form($postmeta)
	{
		$option = $this->m_cache->get_option();
		$this->print_restriction_form($option->get_users(), $postmeta->get_users(),
			DiscloseSecret_POST_USER, DiscloseSecret_META_USER,
			__('Access by user', DiscloseSecret_PLUGIN_ID),
			__('Mark one or more users that should be able to view this post. The post will automatically marked as private and will not be seen by the public if you restrict the access.', DiscloseSecret_PLUGIN_ID),
			__('<ul><li>Highlighted <strong class="current">users</strong> written in bold will at least be able to read your protected and/or private posts.</li><li>Users marked with an exclamation mark <strong class="current">[!]</strong> due to Wordpress will <strong>always</strong> be able to read your protected or private posts, regardless of the settings you will do on this form.</li><li>Users having a role marked with an asterisk mark <strong class="current">[*]</strong> will be able to read your protected posts, because of settings you did for &quot;Access by capability&quot; or &quot;Access by role&quot;.</li></ul>', DiscloseSecret_PLUGIN_ID));
	}

	// Called during the edit form, outputs the Disclose-Secret drop down
	function filter_dbx_post_advanced()
	{
		global $post;

		$postmeta = $this->m_cache->get_postmeta($post->ID);
		$option = $this->m_cache->get_option();
		$option->set_can_see_implicit($postmeta);
		$this->print_user_form($postmeta);
		$this->print_role_form($postmeta);
		$this->print_cap_form($postmeta);
	}

	// ----------------------------------------------------------------------------
	// post overview interface
	// ----------------------------------------------------------------------------

	// Adds a post restriction column to the manage post list
	function filter_manage_posts_columns($columns)
	{
		$columns[DiscloseSecret_PLUGIN_ID] = __('Status', DiscloseSecret_PLUGIN_ID);
		return $columns;
	}

	// Outputs the post restriction for each post in the manage post list
	function filter_manage_posts_custom_column($column_name)
	{
		if ($column_name != DiscloseSecret_PLUGIN_ID)
			return false;

		global $post;
		$postmeta = $this->m_cache->get_postmeta($post->ID);
		if ($postmeta->is_private())
			_e('<strong>Private</strong>', DiscloseSecret_PLUGIN_ID);
		else if ($postmeta->is_protected())
			_e('<em>Protected</em>', DiscloseSecret_PLUGIN_ID);
		else
			_e('Public', DiscloseSecret_PLUGIN_ID);
		return true;
	}
}
endif;

?>