<?php

require_once(dirname(__FILE__).'/config.php');

// ----------------------------------------------------------------------------
// the DiscloseSecret_Migrator class can migrate various plugins to
// DiscloseSecret; also used to migrate its predecessor or older versions
// of this plugin
// ----------------------------------------------------------------------------

class DiscloseSecret_Migrator
{
	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	function DiscloseSecret_Migrator()
	{
	}

	// ----------------------------------------------------------------------------
	// migration functions
	// ----------------------------------------------------------------------------

	function enforce_constraints()
	{
		// just do some clean up to be sure our metas are consistent;
		// after migration it is mandatory but we can do it everywhere
		global $wpdb;

		// selecting duplicate restrictions with equal value
		$query = "
			SELECT `post_id`, `meta_key`, `meta_value`, count(*) AS `count`
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ('". DiscloseSecret_META_CAP. "'
				, '". DiscloseSecret_META_ROLE. "'
				, '". DiscloseSecret_META_USER. "')
			GROUP BY `post_id`, `meta_key`, `meta_value`
			HAVING `count` > 1
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		// deleting duplicates with equal value
		foreach ($s as $i)
		{
			$query = "
				DELETE
				FROM `{$wpdb->postmeta}`
				WHERE `post_id` = {$i->post_id}
				AND `meta_key` = '{$i->meta_key}'
				AND `meta_value` = '{$i->meta_value}'
				LIMIT ". strval($i->count - 1). "
			";
			$wpdb->query($query);
		}
	}

	function migrate_post_restrictions($do_migration)
	{
		global $wpdb;

		// find out whether plugin is installed
		$postrestrictions_installed = false;
		$query = "
			SELECT COUNT(*)
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` = '". DiscloseSecret_POSTRESTRICTIONS_META. "'
		";
		if ($wpdb->get_var($query) > 0)
			$postrestrictions_installed = true;

		if (!$postrestrictions_installed)
			return DiscloseSecret_MIGRATION_NOT_NEEDED;
		if (!do_migration)
			return DiscloseSecret_MIGRATION_SKIPPED;

		$query = "
			UPDATE `{$wpdb->postmeta}`
			SET `meta_key` = '". DiscloseSecret_META_CAP. "'
			WHERE `meta_key` = '". DiscloseSecret_POSTRESTRICTIONS_META. "'
		";
		$wpdb->query($query);

		// option are migrated to new plugin name
		$option = get_option(DiscloseSecret_POSTRESTRICTIONS_PLUGIN_ID);
		update_option(DiscloseSecret_PLUGIN_ID, $option);
		delete_option(DiscloseSecret_POSTRESTRICTIONS_PLUGIN_ID);

		$this->enforce_constraints();
		return DiscloseSecret_MIGRATION_DONE;
	}

	function migrate_post_levels($do_migration)
	{
		global $wpdb, $wp_roles;

		// get the Post Level options
		$postlevels_post_meta_keys[] = DiscloseSecret_POSTLEVELS_META;
		$postlevels_post_meta_key = get_option(DiscloseSecret_POSTLEVELS_META_POST_KEY);
		if ($postlevels_post_meta_key !== false)
			$postlevels_post_meta_keys[] = $postlevels_post_meta_key;
		$postlevels_post_meta_in = "'". implode("', '", $postlevels_post_meta_keys). "'";

		$postlevels_user_meta_keys[] = '';
		$postlevels_user_meta_key = get_option(DiscloseSecret_POSTLEVELS_META_USER_KEY);
		if ($postlevels_user_meta_key !== false)
			$postlevels_user_meta_keys[] = $postlevels_user_meta_key;
		$postlevels_user_meta_in = "'". implode("', '", $postlevels_user_meta_keys). "'";

		// find out whether plugin is installed
		$postlevels_installed = false;
		$query = "
			SELECT COUNT(*)
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($postlevels_post_meta_in)
			AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		if ($wpdb->get_var($query) > 0 && !in_array(DiscloseSecret_META_CAP, $postlevels_post_meta_keys))
			$postlevels_installed = true;

		// find out whether user level migration has been done
		$postlevels_user_migration = false;
		$query = "
			SELECT COUNT(*)
			FROM `{$wpdb->usermeta}`
			WHERE `meta_key` IN ($postlevels_user_meta_in)
			AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		if ($wpdb->get_var($query) > 0)
			$postlevels_user_migration = true;

		if (!$postlevels_installed)
			return DiscloseSecret_MIGRATION_NOT_NEEDED;
		if (!do_migration)
			return DiscloseSecret_MIGRATION_SKIPPED;

		// do migration if requested
		$postlevels_cap_prefix = DiscloseSecret_CAPABILITY_PREFIX_LEVEL;

		// create new capabilities if post levels 1.0 has migrated user levels
		if ($postlevels_user_migration)
		{
			// select user levels to convert to capablities
			$postlevels_cap_prefix = DiscloseSecret_CAPABILITY_PREFIX_READ. $postlevels_cap_prefix;
			$query = "
				SELECT `user_id`, `meta_value`
				FROM `{$wpdb->usermeta}`
				WHERE `meta_key` IN ($postlevels_user_meta_in)
				AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			// create capabilities for this user and administrator role
			foreach ($s as $i)
			{
				// the admin role is needed because some role manager otherwise
				// do not recognize capabilities only assigned to users
				add_role(DiscloseSecret_POSTLEVELS_ROLE_ADMIN,
					__("Post Levels Administrator", DiscloseSecret_PLUGIN_ID));
				$role = get_role(DiscloseSecret_POSTLEVELS_ROLE_ADMIN);
				$role->add_cap($postlevels_cap_prefix. $i->meta_value);
				$user = new WP_User($i->user_id);
				$user->add_cap($postlevels_cap_prefix. $i->meta_value);
			}

			// select post levels to convert to capablities
			$query = "
				SELECT DISTINCT `meta_value`
				FROM `{$wpdb->postmeta}`
				WHERE `meta_key` IN ($postlevels_post_meta_in)
				AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			// create post capabilities for administrator role
			foreach ($s as $i)
			{
				add_role(DiscloseSecret_POSTLEVELS_ROLE_ADMIN,
					__("Post Levels Administrator", DiscloseSecret_PLUGIN_ID));
				$role = get_role(DiscloseSecret_POSTLEVELS_ROLE_ADMIN);
				$role->add_cap($postlevels_cap_prefix. $i->meta_value);
			}

			// delete obsolete post levels user metas
			$query = "
				DELETE
				FROM `{$wpdb->usermeta}`
				WHERE `meta_key` IN ($postlevels_user_meta_in)
			";
			$wpdb->query($query);

			$postlevels_user_migration = false;
		}

		// changing post meta key values to that of Disclose-Secret
		$query = "
			UPDATE `{$wpdb->postmeta}`
			SET `meta_key` = '". DiscloseSecret_META_CAP. "',
				`meta_value` = CONCAT('$postlevels_cap_prefix', `meta_value`)
			WHERE `meta_key` IN ($postlevels_post_meta_in)
			AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$wpdb->query($query);

		$this->enforce_constraint();
		return DiscloseSecret_MIGRATION_DONE;
	}

	function migrate_view_levels($do_migration)
	{
		global $wpdb;

		// get the plugin options
		$viewlevels_post_meta_keys[] = DiscloseSecret_VIEWLEVELS_META;
		$viewlevels_post_meta_in = "'". implode("', '", $viewlevels_post_meta_keys). "'";
		$viewlevels_default = get_option(DiscloseSecret_VIEWLEVELS_OPTION_DEFAULT);
		if ($viewlevels_default === false)
			$viewlevels_default = -1;

		// find out whether plugin is installed
		$viewlevels_installed = false;
		$query = "
			SELECT COUNT(*)
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` IN (-1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		if ($wpdb->get_var($query) > 0)
			$viewlevels_installed = true;

		if (!$viewlevels_installed)
			return DiscloseSecret_MIGRATION_NOT_NEEDED;
		if (!do_migration)
			return DiscloseSecret_MIGRATION_SKIPPED;

		// do migration if requested
		// delete nonfunctional view levels settings
		$query = "
			SELECT `ID` as post_id
			FROM `{$wpdb->posts}`
			WHERE `post_status` NOT IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$unchanged = array();
		foreach ($s as $i)
		{
			$unchanged[] = $i->post_id;
			$query = "
				DELETE
				FROM `{$wpdb->postmeta}`
				WHERE `meta_key` IN ($viewlevels_post_meta_in)
				AND `post_id` = {$i->post_id}
			";
			$wpdb->query($query);
		}
		$unchanged_in = "'". implode("', '", $unchanged). "'";

		// migrate explicitly set protected posts
		$query = "
			SELECT DISTINCT `post_id`
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$protected = array();
		foreach ($s as $i)
		{
			$protected[] = $i->post_id;
			$query = "
				UPDATE `{$wpdb->posts}`
				SET `post_status` = 'private'
				WHERE `post_status` IN ('publish')
				AND `ID` = {$i->post_id}
			";
			$wpdb->query($query);
		}
		$protected_in = "'". implode("', '", $protected). "'";

		$query = "
			UPDATE `{$wpdb->postmeta}`
			SET `meta_key` = '". DiscloseSecret_META_CAP. "',
				`meta_value` = CONCAT('". DiscloseSecret_CAPABILITY_PREFIX_LEVEL. "', `meta_value`)
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$wpdb->query($query);


		// migrate explicitly set public posts
		$query = "
			SELECT DISTINCT `post_id`
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` IN (-1)
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$publish = array();
		foreach ($s as $i)
		{
			// there was also a user level set to restrict access
			// to this post, so we do not make it publish here
			if (in_array($i->post_id, $protected))
				continue;
			$publish[] = $i->post_id;
		}
		$publish_in = "'". implode("', '", $publish). "'";

		$query = "
			DELETE
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
			AND `meta_value` = -1;
		";
		$wpdb->query($query);

		if ($viewlevels_default > -1)
		{
			// migrate default view level
			$query = "
				UPDATE `{$wpdb->posts}`
				SET `post_status` = 'private'
				WHERE `post_status` IN ('publish')
				AND `ID` NOT IN ($unchanged_in)
				AND `ID` NOT IN ($publish_in)
				AND `ID` NOT IN ($protected_in)
			";
			$wpdb->query($query);

			$query = "
				SELECT `ID` AS post_id
				FROM `{$wpdb->posts}`
				WHERE `ID` NOT IN ($unchanged_in)
				AND `ID` NOT IN ($publish_in)
				AND `ID` NOT IN ($protected_in)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			foreach ($s as $i)
			{
				$query = "
					INSERT INTO `{$wpdb->postmeta}`
					(`post_id`, `meta_key`, `meta_value`)
					VALUES ({$i->post_id}, '". DiscloseSecret_META_CAP. "'
						, '". DiscloseSecret_CAPABILITY_PREFIX_LEVEL. "$viewlevels_default')
				";
				$wpdb->query($query);
			}
		}

		// delete obsolete view levels postmetas
		$query = "
			DELETE
			FROM `{$wpdb->postmeta}`
			WHERE `meta_key` IN ($viewlevels_post_meta_in)
		";
		$wpdb->query($query);

		$this->enforce_constraint();
		return DiscloseSecret_MIGRATION_DONE;
	}

	function migrate_private_categories($do_migration)
	{
		global $wpdb;

		// get the plugin options
		$privatecategories_level = get_option(DiscloseSecret_PRIVATECATEGORIES_OPTION_LEVEL);
		if ($privatecategories_level < 0)
			$privatecategories_level = 0;
		else if ($privatecategories_level > 10)
			$privatecategories_level = 10;
		$privatecategories_cats = get_option(DiscloseSecret_PRIVATECATEGORIES_OPTION_CATS);
		$privatecategories_installed = false;
		if (is_array($privatecategories_cats) && count($privatecategories_cats) > 0)
			$privatecategories_installed = true;

		if (!$privatecategories_installed)
			return DiscloseSecret_MIGRATION_NOT_NEEDED;
		if (!do_migration)
			return DiscloseSecret_MIGRATION_SKIPPED;

		// do migration if requested
		$privatecategories_cats_in = "'". implode("', '", $privatecategories_cats). "'";
		$query = "
			SELECT DISTINCT post.`ID` as post_id
			FROM `{$wpdb->posts}` AS post, `{$wpdb->post2cat}` AS post2cat
			WHERE post.`ID` = post2cat.`post_id`
			AND post2cat.`category_id` NOT IN ($privatecategories_cats_in)
			AND `post_status` IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$publish = array();
		foreach ($s as $i)
			$publish[] = $i->post_id;
		$publish_in = "'". implode("', '", $publish). "'";

		$query = "
			SELECT `ID` as post_id
			FROM `{$wpdb->posts}`
			WHERE `post_status` NOT IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$unchanged = array();
		foreach ($s as $i)
			$unchanged[] = $i->post_id;
		$unchanged_in = "'". implode("', '", $unchanged). "'";

		$query = "
			SELECT `ID` AS post_id
			FROM `{$wpdb->posts}`
			WHERE `ID` NOT IN ($unchanged_in)
			AND `ID` NOT IN ($publish_in)
		";
		$wpdb->query($query);
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		foreach ($s as $i)
		{
			$query = "
				INSERT INTO `{$wpdb->postmeta}`
				(`post_id`, `meta_key`, `meta_value`)
				VALUES ({$i->post_id}, '". DiscloseSecret_META_CAP. "'
					, '". DiscloseSecret_CAPABILITY_PREFIX_LEVEL. "$privatecategories_level')
			";
			$wpdb->query($query);
		}

		$query = "
			UPDATE `{$wpdb->posts}`
			SET `post_status` = 'private'
			WHERE `post_status` IN ('publish')
			AND `ID` NOT IN ($unchanged_in)
			AND `ID` NOT IN ($publish_in)
		";
		$wpdb->query($query);

		// cleaning up obsolete stuff
		delete_option(DiscloseSecret_PRIVATECATEGORIES_OPTION_CATS);
		delete_option(DiscloseSecret_PRIVATECATEGORIES_OPTION_LEVEL);

		$this->enforce_constraint();
		return DiscloseSecret_MIGRATION_DONE;
	}
}

?>